@@ -3,6 +3,7 @@ import sys
 from pkg_resources import get_distribution, DistributionNotFound
 
 from . import psmultiprocessing
+from .utils import flatten_results
 
 
 try:
@@ -12,7 +13,10 @@ except DistributionNotFound:  # pragma: no cover
 
 
 def lazy_load_ray():
-    from . import psray
+    try:
+        from . import psray
+    except ImportError:
+        raise ValueError('Import of ray module failed, did you "pip install paramsurvey[ray]"?')
     return {
         'init': psray.init,
         'map': psray.map,
@@ -22,7 +26,10 @@ def lazy_load_ray():
 
 
 def lazy_load_mpi():  # pragma: no cover
-    from . import psmpi
+    try:
+        from . import psmpi
+    except ImportError:
+        raise ValueError('Import of mpi module failed, did you "pip install paramsurvey[mpi]"?')
     return {
         'init': psmpi.init,
         'map': psmpi.map,
@@ -81,5 +88,7 @@ def current_core_count(*args, **kwargs):
 
 def map(*args, **kwargs):
     if our_verbose and 'verbose' not in kwargs:
+        if our_verbose > 1:
+            print('adding verbose= to map kwargs', file=sys.stderr)
         kwargs['verbose'] = our_verbose
     return our_backend['map'](*args, **kwargs)
