@@ -9,6 +9,7 @@ import pyarrow
 
 from . import utils
 from . import stats
+from .utils import ResultsObject
 
 
 def read_ray_config():
@@ -78,7 +79,7 @@ def do_work_wrapper(func, system_kwargs, user_kwargs, psets):
                 result = func(pset, system_kwargs, user_kwargs, raw_stats)
             user_ret['result'] = result
         except Exception as e:
-            user_ret['exception'] = str(e)
+            user_ret['exception'] = repr(e)
             print('saw an exception in the worker function', file=sys.stderr)
             print('it was working on', json.dumps(pset, sort_keys=True), file=sys.stderr)
             traceback.print_exc()
@@ -87,6 +88,8 @@ def do_work_wrapper(func, system_kwargs, user_kwargs, psets):
 
 
 def handle_return(out_func, ret, system_stats, system_kwargs, user_kwargs):
+    progress = system_kwargs['progress']
+
     try:
         ret = ray.get(ret)
     except Exception as e:
@@ -95,22 +98,29 @@ def handle_return(out_func, ret, system_stats, system_kwargs, user_kwargs):
               'an unknown number of results lost\n'.format(e), file=sys.stderr)
         traceback.print_exc()
         sys.stderr.flush()
-        progress = system_kwargs['progress']
         progress['failures'] += 1
         utils.report_progress(system_kwargs)
         return
 
-    progress = system_kwargs['progress']
-    progress['retired'] += len(ret)
-
     for user_ret, system_ret in ret:
         if 'result' in user_ret and not isinstance(user_ret['result'], dict) and user_ret['result'] is not None:
-            raise ValueError('user function did not return a dict')
-        out_func(user_ret, system_kwargs, user_kwargs)
+            # fake an exception, make this case look like other failures
+            user_ret['exception'] = "ValueError('user function did not return a dict: {}')".format(
+                repr(user_ret['result']))
+            user_ret['result'] = {}
         if 'raw_stats' in system_ret:
             system_stats.combine_stats(system_ret['raw_stats'])
+        pset_id = user_ret['pset']['_pset_id']
         if 'exception' in user_ret:
             progress['failures'] += 1
+            progress['exceptions'] += 1
+            system_kwargs['pset_ids'][pset_id]['exception'] = user_ret['exception']
+        else:
+            del system_kwargs['pset_ids'][pset_id]
+            system_kwargs['results'].append(user_ret)
+            progress['finished'] += len(ret)
+        if out_func:
+            out_func(user_ret, system_kwargs, user_kwargs)
 
     utils.report_progress(system_kwargs)
 
@@ -147,14 +157,12 @@ def progress_until_fewer(futures, cores, factor, out_func, system_stats, system_
     return futures, cores, group_size
 
 
-def map(func, psets, out_func=utils.accumulate_return, user_kwargs=None, chdir=None, outfile=None, out_subdirs=None,
-        progress_dt=60., group_size=None, name='pset', verbose=None, **kwargs):
+def map(func, psets, out_func=None, user_kwargs=None, chdir=None, outfile=None, out_subdirs=None,
+        progress_dt=60., group_size=None, name='default', verbose=None, **kwargs):
     if not psets:
         return
 
-    psets = psets.copy()  # we are going to be popping it
-
-    system_stats, system_kwargs = utils.map_prep(name, chdir, outfile, out_subdirs, len(psets), verbose, **kwargs)
+    psets, system_stats, system_kwargs = utils.map_prep(psets, name, chdir, outfile, out_subdirs, verbose, **kwargs)
 
     progress = system_kwargs['progress']
     cores = current_core_count()
@@ -165,7 +173,9 @@ def map(func, psets, out_func=utils.accumulate_return, user_kwargs=None, chdir=N
         group_size = 1
 
     if verbose:
-        print('starting map, inital core count is', cores, file=sys.stderr)
+        print('starting map, {} psets {} cores {} group_size'.format(
+            len(psets), cores, group_size
+        ), file=sys.stderr)
         sys.stderr.flush()
 
     futures = []
@@ -187,9 +197,10 @@ def map(func, psets, out_func=utils.accumulate_return, user_kwargs=None, chdir=N
     if verbose:
         print('finished getting results', file=sys.stderr)
         sys.stderr.flush()
+
+    utils.finalize_progress(system_kwargs)
     utils.report_progress(system_kwargs, final=True)
 
     system_stats.print_histograms(name)
 
-    if 'user_ret' in system_kwargs:
-        return system_kwargs['user_ret']
+    return ResultsObject(system_kwargs['results'], list(system_kwargs['pset_ids'].values()), system_kwargs['progress'], system_stats)
