@@ -2,22 +2,11 @@ import time
 import os
 import tempfile
 import pytest
+import sys
 
 import paramsurvey
 import paramsurvey.stats
-
-
-def do_sleep(pset, system_kwargs, user_kwargs, raw_stats):
-    time.sleep(pset['duration'])
-    return {'slept': pset['duration']}
-
-
-def do_burn(pset, system_kwargs, user_kwargs, raw_stats):
-    start = time.time()
-    with paramsurvey.stats.record_wallclock('foo', raw_stats):
-        while time.time() < start + pset['duration']:
-            pass
-    return {'burned': pset['duration']}
+from paramsurvey.examples import sleep_worker, burn_worker
 
 
 @pytest.fixture(scope="module")
@@ -38,23 +27,23 @@ def test_basics(paramsurvey_init):
     psets = [{'duration': duration}] * ncores * 4
 
     start = time.time()
-    ret = paramsurvey.map(do_sleep, psets, name='simple')
-    assert [r['result']['slept'] == duration for r in ret], 'everyone slept '+str(duration)
-    assert len(ret) == len(psets), 'one return for each pset'
+    results = paramsurvey.map(sleep_worker, psets, name='simple')
+    assert [r['result']['slept'] == duration for r in results.results], 'everyone slept '+str(duration)
+    assert len(results.results) == len(psets), 'one return for each pset'
     elapsed = time.time() - start
     assert elapsed > duration, 'must take at least {} time'.format(duration)
 
     psets = psets[:ncores]
     start = time.time()
-    ret = paramsurvey.map(do_sleep, psets, name='group_size 5', group_size=5)
-    assert [r['result']['slept'] == duration for r in ret], 'everyone slept '+str(duration)
-    assert len(ret) == len(psets), 'one return for each pset'
+    results = paramsurvey.map(sleep_worker, psets, name='group_size 5', group_size=5)
+    assert [r['result']['slept'] == duration for r in results.results], 'everyone slept '+str(duration)
+    assert len(results.results) == len(psets), 'one return for each pset'
     elapsed = time.time() - start
     assert elapsed > duration*3, 'must take at least {} time'.format(duration)
 
-    ret = paramsurvey.map(do_burn, psets, name='burn group_size 5', group_size=4)
-    assert [r['result']['burned'] == duration for r in ret], 'everyone burned '+str(duration)
-    assert len(ret) == len(psets), 'one return for each pset'
+    results = paramsurvey.map(burn_worker, psets, name='burn group_size 5', group_size=4)
+    assert [r['result']['burned'] == duration for r in results.results], 'everyone burned '+str(duration)
+    assert len(results.results) == len(psets), 'one return for each pset'
     elapsed = time.time() - start
     assert elapsed > duration*3, 'must take at least {} time'.format(duration)
 
@@ -74,7 +63,6 @@ def test_args(capsys, paramsurvey_init):
     out_func_called = False
     test_user_kwargs = {'test': 1, 'expected_cwd': chdir}
     outfile = 'foo'
-    name = 'foobiebletch'
 
     def out_func(user_ret, system_kwargs, user_kwargs):
         nonlocal out_func_called
@@ -86,16 +74,19 @@ def test_args(capsys, paramsurvey_init):
 
     psets = [{'duration': 0.1}] * 2
 
-    ret = paramsurvey.map(do_test_args, psets,
-                          out_func=out_func, user_kwargs=test_user_kwargs,
-                          chdir=chdir, outfile=outfile, out_subdirs=10,
-                          progress_dt=0., name=name)
+    name = 'test_args dt=0'
+    results = paramsurvey.map(do_test_args, psets,
+                              out_func=out_func, user_kwargs=test_user_kwargs,
+                              chdir=chdir, outfile=outfile, out_subdirs=10,
+                              progress_dt=0., name=name)
 
     assert out_func_called
     assert test_user_kwargs.get('out_func_called')
-    assert ret is None  # because of out_func
+    assert len(results.results) == 2
 
     captured = capsys.readouterr()
+    sys.stdout.write(captured.out)
+    sys.stderr.write(captured.err)
     assert len(captured.err.splitlines()) >= len(psets)
 
     # because of progress_dt being 0., we should have at least len(psets) progress lines
@@ -103,24 +94,27 @@ def test_args(capsys, paramsurvey_init):
     assert len(has_name) >= len(psets)
 
     # same as previous but verbose=2 instead of progress_dt=0
-    ret = paramsurvey.map(do_test_args, psets,
-                          out_func=out_func, user_kwargs=test_user_kwargs,
-                          chdir=chdir, outfile=outfile, out_subdirs=10,
-                          verbose=2, name=name)
+    name = 'test_args verbose=2'
+    results = paramsurvey.map(do_test_args, psets,
+                              out_func=out_func, user_kwargs=test_user_kwargs,
+                              chdir=chdir, outfile=outfile, out_subdirs=10,
+                              verbose=2, name=name)
 
     assert out_func_called
     assert test_user_kwargs.get('out_func_called')
-    assert ret is None  # because of out_func
+    assert len(results.results) == 2
 
     captured = capsys.readouterr()
+    sys.stdout.write(captured.out)
+    sys.stderr.write(captured.err)
     assert len(captured.err.splitlines()) >= len(psets)
 
     # because of progress_dt being 0., we should have at least len(psets) progress lines
     has_name = [line for line in captured.err.splitlines() if 'progress' in line and name in line]
     assert len(has_name) >= len(psets)
 
-    ret = paramsurvey.map(do_test_args, [])
-    assert ret is None
+    results = paramsurvey.map(do_test_args, [], name='no psets')
+    assert results is None
 
 
 def do_raise(pset, system_kwargs, user_kwargs, raw_stats):
@@ -132,20 +126,28 @@ def do_raise(pset, system_kwargs, user_kwargs, raw_stats):
 def test_worker_exception(capsys, paramsurvey_init):
     psets = [{}, {}, {}, {'raise': True}, {}, {}, {}]
 
-    ret = paramsurvey.map(do_raise, psets)
-    assert len(ret) == 7
-    assert sum('exception' in r for r in ret if r is not None) == 1
-    assert sum('pset' in r for r in ret) == 7
-    assert sum('result' in r for r in ret) == 6
+    results = paramsurvey.map(do_raise, psets, name='test_worker_exception')
+    assert len(results.results) == 6
+    assert len(results.missing) == 1
+    assert 'exception' in results.missing[0]
+    assert results.progress.total == 7
+    assert results.progress.finished == 6
+    assert results.progress.failures == 1
+    assert results.progress.exceptions == 1
 
-    out, err = capsys.readouterr()
+    assert sum('pset' in r for r in results.results) == 6
+    assert sum('result' in r for r in results.results) == 6
+
+    captured = capsys.readouterr()
+    sys.stdout.write(captured.out)
+    sys.stderr.write(captured.err)
 
     # ray redirects stderr to stdout, while multiprocessing prints it in the worker
     # TODO: add stderr/out capture everywhere and use it here
-    #assert 'Traceback ' in out or 'Traceback ' in err
+    #assert 'Traceback ' in captured.out or 'Traceback ' in captured.err
 
     # the standard progress function prints this
-    assert 'failures: 1' in out or 'failures: 1' in err
+    assert 'failures: 1' in captured.out or 'failures: 1' in captured.err
 
 
 def do_nothing(pset, system_kwargs, user_kwargs, raw_stats):
@@ -153,22 +155,28 @@ def do_nothing(pset, system_kwargs, user_kwargs, raw_stats):
 
 
 def test_wrapper_exception(capsys, paramsurvey_init):
-    psets = [{}, {'actually_raise': True}, {}, {}, {}]
+    psets = [{}, {'actually_raise': True}, {}, {'actually_raise': True}, {}]
 
-    ret = paramsurvey.map(do_nothing, psets, raise_in_wrapper=ValueError('test_wrapper_exception'))
+    results = paramsurvey.map(do_nothing, psets, raise_in_wrapper=ValueError('test_wrapper_exception'))
 
-    # ray eats a return when it raises
-    assert len(ret) == 5 or len(ret) == 4
-    assert sum('result' in r for r in ret) == 4  # same for ray and multi
-    if len(ret) == 5:
-        assert sum('exception' in r for r in ret) == 1
-    else:
-        assert sum('exception' in r for r in ret) == 0
+    assert len(results.results) == 3
+    assert len(results.missing) == 2
+    assert results.progress.total == 5
+    assert results.progress.finished == 3
+    assert results.progress.failures == 2
+
+    # ray and multiprocessing behave differently for 'exception'
+    #assert 'exception' in results.missing[0]
+    #assert results.progress.exceptions == 1
+
+    assert sum('result' in r for r in results.results) == 3
 
     # XXX ray prints traceback in the worker, multiprocessing and ray local_mode prints in the parent
 
-    out, err = capsys.readouterr()
-    assert 'failures: 1' in out or 'failures: 1' in err
+    captured = capsys.readouterr()
+    sys.stdout.write(captured.out)
+    sys.stderr.write(captured.err)
+    assert 'failures: 2' in captured.out or 'failures: 2' in captured.err
 
 
 def test_overlarge_pset():
